from django.db import models
from django.utils.text import gettext_lazy as _

from side.models import BaseModel
from .managers import (CodeRuleManager, SampleManager, SampleOperationManager,
                       RefrigeratorManager, AreaManager, FrozenFrameManager, FrozenBoxManager)


class CodeRule(BaseModel):
    tag = models.CharField(_('tag'), max_length=50, unique=True)
    segments = models.JSONField(_('segments'))
    segment_order = models.TextField(_('segment order'))

    objects = CodeRuleManager()

    class Meta:
        db_table = 'code_rules'
        verbose_name = _('code rule')
        verbose_name_plural = _('code rules')


class Sample(BaseModel):
    """
    A model Sample.
    """
    SAMPLE_STATUS_CHOICES = [
        ('ND', _('Not Deposited')),
        ('FD', _('First Deposited')),
        ('CO', _('Checkout')),
        ('RD', _('Redeposited'))
    ]
    GENDERS_CHOICES = [
        ('U', _('Unknown')),
        ('M', _('Male')),
        ('F', _('Female')),
        ('T', _('Third Gender'))
    ]

    code = models.CharField(_('sample code'), max_length=250, blank=True, null=True)
    code_rule = models.CharField(_('sample code rule'), max_length=250, blank=True, null=True)
    qr_code = models.CharField(_('sample QR code'), max_length=255, blank=True, null=True)
    status = models.CharField(_('sample status'), max_length=10,
                              choices=SAMPLE_STATUS_CHOICES, default='ND')
    freeze_thawing_cycles = models.IntegerField(_('freeze-thawing cycles'), default=0)
    category = models.CharField(_('sample category'), max_length=50, blank=True, null=True)
    from_where = models.TextField(_('sample from where'), blank=True, null=True)
    from_whom = models.CharField(_('sample from whom'), max_length=250, default=_('unknown'))
    gender = models.CharField(_('gender'), max_length=2, choices=GENDERS_CHOICES, default='U')
    age = models.IntegerField(_('age'), blank=True, null=True)
    hospital_admission_number = models.CharField(_('hospital admission number'), max_length=50,
                                                 blank=True, null=True)
    frozen_box_id = models.CharField(_('frozen box'),
                                     max_length=36, blank=True, null=True)
    frozen_box_type = models.CharField(_('frozen box type'),
                                       max_length=36, blank=True, null=True)
    frozen_box_point = models.CharField(_('frozen box point'), max_length=100, blank=True, null=True)
    stored_in = models.TextField(_('sample stored in'), blank=True, null=True)
    first_storage_date = models.DateTimeField(_('first storage date'), auto_now_add=True)
    last_storage_date = models.DateTimeField(_('last storage date'), auto_now=True)
    description = models.TextField(_('description'), blank=True, null=True)

    objects = SampleManager()

    class Meta:
        db_table = 'bio_samples'
        verbose_name = _('sample')
        verbose_name_plural = _('samples')


class SampleOperation(BaseModel):
    SAMPLE_OPERATION_CHOICES = [
        ('CI', _('Check In')),
        ('CO', _('Check Out')),
        ('RD', _('Redeposit'))
    ]

    sample_id = models.CharField(_('sample id'), max_length=64)
    sample_code = models.CharField(_('sample code'), max_length=36)
    stored_in = models.TextField(_('sample stored in'), blank=True, null=True)
    operation = models.CharField(_('sample operation'), max_length=2,
                                 choices=SAMPLE_OPERATION_CHOICES, default='CI')
    operator = models.CharField(_('sample operator'), max_length=36)
    operation_date = models.DateTimeField(_('sample operation time'), auto_now_add=True)
    description = models.TextField(_('description'), blank=True, null=True)

    objects = SampleOperationManager()

    class Meta:
        db_table = 'bio_sample_operations'
        verbose_name = _('sample operation')
        verbose_name_plural = _('sample operations')


class Area(BaseModel):
    name = models.CharField(_('area name'), max_length=250)
    parent_id = models.ForeignKey('self',
                                  db_column='parent_id',
                                  on_delete=models.CASCADE,
                                  null=True,
                                  related_name='subareas')

    objects = AreaManager()

    class Meta:
        db_table = 'bio_areas'
        verbose_name = _('area')
        verbose_name_plural = _('areas')


class Refrigerator(BaseModel):
    REFRIGERATOR_TYPES = (
        ('L', _('layered type')),
        ('C', _('container type'))
    )
    area = models.ForeignKey(Area,
                             related_name='refrigerators',
                             on_delete=models.SET_NULL,
                             blank=True,
                             null=True)
    name = models.CharField(_('refrigerator name'), max_length=250)
    type = models.CharField(_('refrigerator type'), max_length=2,
                            choices=REFRIGERATOR_TYPES,
                            default='L')
    tag = models.CharField(_('refrigerator category'), max_length=250, blank=True, null=True)
    description = models.TextField(_('description'), blank=True, null=True)

    objects = RefrigeratorManager()

    class Meta:
        db_table = 'bio_refrigerators'
        unique_together = ('area_id', 'name')
        verbose_name = _('refrigerator')
        verbose_name_plural = _('refrigerators')


class FrozenFrame(BaseModel):
    refrigerator = models.ForeignKey(Refrigerator,
                                     related_name='frozen_frames',
                                     on_delete=models.SET_NULL,
                                     blank=True,
                                     null=True)
    name = models.CharField(_('frozen frame name'), max_length=250)
    tag = models.CharField(_('frozen frame category'), max_length=250, blank=True, null=True)
    rows = models.IntegerField(_('rows'), default=10)
    cols = models.IntegerField(_('cols'), default=10)

    objects = FrozenFrameManager()

    class Meta:
        db_table = 'bio_frozen_frames'
        unique_together = ('refrigerator_id', 'name')
        verbose_name = _('frozen_frame')
        verbose_name_plural = _('frozen_frames')


class FrozenBox(BaseModel):
    frozen_frame = models.ForeignKey(FrozenFrame,
                                     related_name='frozen_boxes',
                                     on_delete=models.SET_NULL,
                                     blank=True,
                                     null=True)
    name = models.CharField(_('frozen box name'), max_length=250)
    capacity = models.CharField(_('frozen box capacity'), max_length=250, blank=True, null=True)
    rows = models.IntegerField(_('rows'), default=10)
    cols = models.IntegerField(_('cols'), default=10)

    objects = FrozenBoxManager()

    class Meta:
        db_table = 'bio_frozen_boxes'
        unique_together = ('frozen_frame', 'name')
        verbose_name = _('frozen_box')
        verbose_name_plural = _('frozen_boxes')