from django.utils.text import gettext_lazy as _
from drf_yasg import openapi
from rest_framework import serializers

from side.settings import DEFAULT_SERIALIZER_EXCLUDE, DEFAULT_SERIALIZER_READ_ONLY

from .utils import get_real_code
from .models import (CodeRule, Sample, SampleOperation, Refrigerator,
                     FrozenFrame, FrozenBox, Area)
from .fields import AreaDefault, SubareaDefault, RefrigeratorDefault, FrozenFrameDefault


class CodeRuleSegmentField(serializers.JSONField):

    class Meta:
        swagger_schema_fields = {
            'type': openapi.TYPE_OBJECT,
            'title': _('Code Rule Segments'),
            'properties': {
                'segment_key': openapi.Schema(
                    title=_('Segment key'),
                    type=openapi.TYPE_STRING,
                ),
                'segment_value': openapi.Schema(
                    title=_('Segment initial value'),
                    type=openapi.TYPE_STRING,
                ),
                'segment_type': openapi.Schema(
                    title=_('Segment value type'),
                    type=openapi.TYPE_STRING,
                    enum=['constant', 'random', 'date', 'placeholder'],
                ),
                'segment_type_random_size': openapi.Schema(
                    title=_('Segment type random size'),
                    type=openapi.TYPE_INTEGER,
                    default=4,
                ),
                'segment_type_date_display': openapi.Schema(
                    title=_('''Date display, %y is year(00-99), %Y is year(000-9999),
                    %m is month(01-12), %d is day(0-31), and it can be used in combination.'''),
                    type=openapi.TYPE_STRING,
                    enum=['%y', '%Y', '%m', '%d'],
                    default='%y'
                ),
            },
            'required': ['segment_key', 'segment_value', 'segment_type'],
        }


class CodeRuleSegmentListField(serializers.ListField):
    child = CodeRuleSegmentField()


class CodeRuleSerializer(serializers.ModelSerializer):
    tag = serializers.CharField(max_length=50)
    segments = CodeRuleSegmentListField()

    class Meta:
        model = CodeRule
        exclude = DEFAULT_SERIALIZER_EXCLUDE
        read_only_fields = DEFAULT_SERIALIZER_READ_ONLY


class SampleSerializer(serializers.ModelSerializer):

    def create(self, validated_data):
        code = validated_data.pop('code')
        validated_data['code'] = get_real_code(code)
        sample = Sample.objects.create(**validated_data)
        return sample

    class Meta:
        model = Sample
        exclude = DEFAULT_SERIALIZER_EXCLUDE
        read_only_fields = DEFAULT_SERIALIZER_READ_ONLY


class SampleListSerializer(serializers.ListSerializer):
    child = SampleSerializer()

    def create(self, validated_data):
        samples = []
        for item in validated_data:
            item['code'] = get_real_code(item['code'])
            samples.append(item)
        samples = [Sample(**item) for item in validated_data]
        return Sample.objects.bulk_create(samples)


class SampleBulkDestroySerializer(serializers.Serializer):
    sample_ids = serializers.ListField()


class SampleOperationSerializer(serializers.ModelSerializer):
    class Meta:
        model = SampleOperation
        exclude = DEFAULT_SERIALIZER_EXCLUDE
        read_only_fields = DEFAULT_SERIALIZER_READ_ONLY


class RecursiveSerializer(serializers.Serializer):
    def to_representation(self, value):
        serializer = self.parent.parent.__class__(value, context=self.context)
        return serializer.data


class AreaSerializer(serializers.ModelSerializer):
    subareas = RecursiveSerializer(many=True, read_only=True)
    refrigerators = serializers.StringRelatedField(many=True, read_only=True)

    class Meta:
        model = Area
        exclude = DEFAULT_SERIALIZER_EXCLUDE
        read_only_fields = DEFAULT_SERIALIZER_READ_ONLY + ['parent_id']


class SubareaSerializer(serializers.ModelSerializer):
    parent_id = serializers.HiddenField(default=SubareaDefault())

    class Meta:
        model = Area
        exclude = DEFAULT_SERIALIZER_EXCLUDE
        read_only_fields = DEFAULT_SERIALIZER_READ_ONLY


class RefrigeratorSerializer(serializers.ModelSerializer):
    area = serializers.HiddenField(default=AreaDefault())
    frozen_frames = serializers.StringRelatedField(many=True, read_only=True)

    class Meta:
        model = Refrigerator
        exclude = DEFAULT_SERIALIZER_EXCLUDE
        read_only_fields = DEFAULT_SERIALIZER_READ_ONLY


class FrozenFrameSerializer(serializers.ModelSerializer):
    refrigerator = serializers.HiddenField(default=RefrigeratorDefault())
    frozen_boxes = serializers.StringRelatedField(many=True, read_only=True)

    class Meta:
        model = FrozenFrame
        exclude = DEFAULT_SERIALIZER_EXCLUDE
        read_only_fields = DEFAULT_SERIALIZER_READ_ONLY


class FrozenBoxSerializer(serializers.ModelSerializer):
    frozen_frame = serializers.HiddenField(allow_null=True, default=FrozenFrameDefault())

    class Meta:
        model = FrozenBox
        exclude = DEFAULT_SERIALIZER_EXCLUDE
        read_only_fields = DEFAULT_SERIALIZER_READ_ONLY



