import uuid

from django.db import models
from django.utils.text import gettext_lazy as _

from side.models import BaseModel
from side.data.managers import TagManager, FormManager, VocabManager, EntryManager, ConfigManager


class Tag(BaseModel):
    key = models.CharField(_('key'), max_length=150, unique=True)
    label = models.CharField(_('label'), max_length=150, unique=True)

    objects = TagManager()

    class Meta:
        db_table = 'data_tags'
        verbose_name = _('data tag')
        verbose_name_plural = _('data tags')


class Form(BaseModel):
    title = models.CharField(_('title'), max_length=200, blank=True, null=True)
    tag = models.CharField(_('tag'), max_length=200, default='default', unique=True)
    description = models.TextField(_('description'), blank=True, null=True)
    fields = models.JSONField(_('field'), blank=True, null=True)

    objects = FormManager()

    class Meta:
        db_table = 'data_forms'
        verbose_name = _('data form')
        verbose_name_plural = _('data forms')

    def __str__(self):
        return '%s: %s' % (self.title, self.tag)


class Vocab(BaseModel):
    name = models.CharField(_('name'), max_length=200,
                            help_text=_('Specifies the vocab name.'))
    tag = models.CharField(_('tag'), max_length=150, unique=True)
    description = models.TextField(_('description'), blank=True, null=True)

    objects = VocabManager()

    class Meta:
        db_table = 'data_vocabs'
        verbose_name = _('data vocab')
        verbose_name_plural = _('data vocabs')

    def __str__(self):
        return '%s: %s' % (self.name, self.tag)


class VocabEntry(models.Model):
    vocab_id = models.ForeignKey(Vocab, related_name='entries',
                                 db_column='vocab_id',
                                 on_delete=models.CASCADE)
    entry_key = models.CharField(_('entry key'), max_length=150)
    entry_label = models.CharField(_('entry label'), max_length=150)
    entry_links = models.TextField(_('entry links'), blank=True, null=True)
    extras = models.JSONField(_('extra data'), blank=True, null=True)
    deleted_at = models.DateTimeField(blank=True, null=True, default=None, editable=False, db_index=True)

    objects = EntryManager()

    class Meta:
        db_table = 'data_vocab_entries'
        verbose_name = _('data vocab entry')
        verbose_name_plural = _('data vocab entries')

    def __str__(self):
        return '%s: %s' % (self.entry_key, self.entry_label)


class Config(BaseModel):
    name = models.CharField(_('config name'), max_length=200,
                            help_text=_('Specifies the config name.'))
    tag = models.CharField(_('config tag'), max_length=150, unique=True)
    items = models.JSONField(_('config items'), blank=True, null=True)
    description = models.TextField(_('description'), blank=True, null=True)

    objects = ConfigManager()

    class Meta:
        db_table = 'data_configs'
        verbose_name = _('data config')
        verbose_name_plural = _('data configs')

    def __str__(self):
        return '%s: %s' % (self.name, self.tag)


class Extra(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    project_id = models.CharField(_('project'), max_length=64, blank=True, null=True)
    tag = models.CharField(_('tag'), max_length=150)
    fields = models.JSONField(_('field'), blank=True, null=True)
    created_at = models.DateTimeField(_('date created'), auto_now_add=True)
    updated_at = models.DateTimeField(_('date updated'), auto_now=True)
    deleted_at = models.DateTimeField(_('date deleted'), blank=True, null=True, default=None,
                                      editable=False, db_index=True)

    class Meta:
        db_table = 'data_extras'
        verbose_name = _('data extra')
        verbose_name_plural = _('data extras')