from django.utils.text import gettext_lazy as _
from drf_yasg import openapi
from rest_framework import serializers

from side.settings import DEFAULT_SERIALIZER_EXCLUDE, DEFAULT_SERIALIZER_READ_ONLY
from side.data.models import Tag, Form, Vocab, VocabEntry, Config, Extra


class TagSerializer(serializers.ModelSerializer):

    class Meta:
        model = Tag
        exclude = DEFAULT_SERIALIZER_EXCLUDE
        read_only_fields = DEFAULT_SERIALIZER_READ_ONLY


class TagListSerializer(serializers.ListSerializer):
    child = TagSerializer()

    def create(self, validated_data):
        tags = [Tag(**item) for item in validated_data]
        return Tag.objects.bulk_create(tags)


class TagBulkDestroySerializer(serializers.Serializer):
    tag_ids = serializers.ListField()


class FieldJSONField(serializers.JSONField):
    class Meta:
        swagger_schema_fields = {
            'type': openapi.TYPE_OBJECT,
            'title': _('Form Fields'),
            'properties': {
                'field_key': openapi.Schema(
                    title=_('field key'),
                    type=openapi.TYPE_STRING,
                ),
                'field_value': openapi.Schema(
                    title=_('field initial value'),
                    type=openapi.TYPE_STRING,
                ),
                'field_type': openapi.Schema(
                    title=_('field value type'),
                    type=openapi.TYPE_STRING,
                ),
                'field_required': openapi.Schema(
                    title=_('field required'),
                    type=openapi.TYPE_INTEGER,
                    enum=[0, 1],
                ),
                'field_label': openapi.Schema(
                    title=_('field label'),
                    type=openapi.TYPE_STRING,
                ),
                'field_read_only': openapi.Schema(
                    title=_('field read only'),
                    type=openapi.TYPE_INTEGER,
                    enum=[0, 1],
                ),
            },
            'required': ['field_key', 'field_value', 'field_type', 'field_required', 'field_label'],
        }


class FieldListField(serializers.ListField):
    child = FieldJSONField()


class FormSerializer(serializers.ModelSerializer):
    fields = FieldListField()

    class Meta:
        model = Form
        exclude = DEFAULT_SERIALIZER_EXCLUDE
        read_only_fields = DEFAULT_SERIALIZER_READ_ONLY


class ExtraSerializer(serializers.ModelSerializer):
    fields = FieldListField()

    class Meta:
        model = Extra
        exclude = DEFAULT_SERIALIZER_EXCLUDE
        read_only_fields = DEFAULT_SERIALIZER_READ_ONLY


class VocabSerializer(serializers.ModelSerializer):
    entries = serializers.StringRelatedField(many=True, read_only=True)

    class Meta:
        model = Vocab
        exclude = DEFAULT_SERIALIZER_EXCLUDE
        read_only_fields = DEFAULT_SERIALIZER_READ_ONLY


class VocabDefault:

    requires_context = True

    def __call__(self, serializer_field):
        vocab_id = serializer_field.context['request'].data.get('vocab_id', None)
        if vocab_id:
            return Vocab.objects.get_by_natural_key(vocab_id)
        else:
            return None

    def __repr__(self):
        return '%s()' % self.__class__.__name__


class VocabEntrySerializer(serializers.ModelSerializer):
    vocab_id = serializers.HiddenField(default=VocabDefault())

    class Meta:
        model = VocabEntry
        fields = ('id', 'vocab_id', 'entry_key', 'entry_label', 'entry_links', 'extras')


class ConfigSerializer(serializers.ModelSerializer):

    class Meta:
        model = Config
        exclude = DEFAULT_SERIALIZER_EXCLUDE
        read_only_fields = DEFAULT_SERIALIZER_READ_ONLY
