import logging

from django.db import connection

from rest_framework_tracking.base_mixins import BaseLoggingMixin

from side.identity.models import Project
from side.monitoring.models import APIRequest

logger = logging.getLogger(__name__)


class APIRequestMixin(BaseLoggingMixin):
    def handle_log(self):
        """
        Hook to define what happens with the request.

        Defaults on saving the data on the db.
        """
        APIRequest(**self.log).save()

    def finalize_response(self, request, response, *args, **kwargs):
        response = super(BaseLoggingMixin, self).finalize_response(
            request, response, *args, **kwargs
        )
        # Ensure backward compatibility for those using _should_log hook
        should_log = (
            self._should_log if hasattr(self, "_should_log") else self.should_log
        )
        if should_log(request, response):
            if (
                connection.settings_dict.get("ATOMIC_REQUESTS") and getattr(response, "exception", None) and connection.in_atomic_block
            ):
                # response with exception (HTTP status like: 401, 404, etc)
                # pointwise disable atomic block for handle log (TransactionManagementError)
                connection.set_rollback(True)
                connection.set_rollback(False)
            if response.streaming:
                rendered_content = None
            elif hasattr(response, "rendered_content"):
                rendered_content = response.rendered_content
            else:
                rendered_content = response.getvalue()

            self.log.update(
                {
                    "project_id": self._get_project_id(request),
                    "project_name": self._get_project_name(request),
                    "remote_addr": self._get_ip_address(request),
                    "view": self._get_view_name(request),
                    "view_method": self._get_view_method(request),
                    "path": request.path,
                    "host": request.get_host(),
                    "method": request.method,
                    "query_params": self._clean_data(request.query_params.dict()),
                    "user": self._get_user(request),
                    "username_persistent": self._get_user(request).get_username()
                    if self._get_user(request)
                    else "Anonymous",
                    "response_ms": self._get_response_ms(),
                    "response": self._clean_data(rendered_content),
                    "status_code": response.status_code,
                }
            )
            if self._clean_data(request.query_params.dict()) == {}:
                self.log.update({"query_params": self.log["data"]})
            try:
                self.handle_log()
            except Exception:
                # ensure that all exceptions raised by handle_log
                # doesn't prevent API call to continue as expected
                logger.exception("Logging API call raise exception!")

        return response

    def _get_project_id(self, request):
        if hasattr(request.user, 'default_project_id'):
            return request.user.default_project_id
        else:
            return None

    def _get_project_name(self, request):
        if hasattr(request.user, 'default_project_id'):
            project_id = request.user.default_project_id
            project = Project.objects.get_by_natural_key(pk=project_id)
            return project.name
        else:
            return None


class APIRequestErrorsMixin(APIRequestMixin):
    """
    API Request only errors
    """

    def should_log(self, request, response):
        return response.status_code >= 400