from drf_yasg import openapi
from drf_yasg.utils import swagger_auto_schema

from rest_framework import viewsets
from rest_framework.permissions import IsAuthenticated
from rest_framework.decorators import action
from rest_framework.generics import GenericAPIView
from rest_framework.response import Response

from side.monitoring.mixins import APIRequestMixin
from side.nlp.serializers import (TextSerializer, OneMatchManySerializer,
                                  ManyMatchOneSerializer, SearchSimilaritySerializer)
from side.nlp.matchers.regex import RegexMatcher
from side.nlp.matchers.fuzzy import FuzzyMatcher
from side.nlp.matchers.similarity import search_similarity
from side.nlp.backends.lac.features import ner as lac_ner


class FeatureViewSets(viewsets.ViewSet, APIRequestMixin, GenericAPIView):
    permission_classes = [IsAuthenticated]

    lang = openapi.Parameter('lang',
                             openapi.IN_QUERY,
                             type=openapi.TYPE_STRING,
                             required=True,
                             default='zh')
    threshold = openapi.Parameter('threshold',
                                  openapi.IN_QUERY,
                                  type=openapi.TYPE_NUMBER,
                                  default=0.3)
    mode = openapi.Parameter('mode',
                             openapi.IN_QUERY,
                             type=openapi.TYPE_STRING,
                             required=True,
                             enum=['simple', 'partial', 'token_sort', 'token_set'],
                             default='simple')
    return_all = openapi.Parameter('return_all',
                                   openapi.IN_QUERY,
                                   required=True,
                                   type=openapi.TYPE_BOOLEAN,
                                   default=False)
    use_default_model = openapi.Parameter('use_default_model',
                                          openapi.IN_QUERY,
                                          type=openapi.TYPE_BOOLEAN,
                                          required=True,
                                          default=True)
    custom_model_id = openapi.Parameter('custom_model_id',
                                        openapi.IN_QUERY,
                                        type=openapi.FORMAT_UUID,
                                        required=False)

    @swagger_auto_schema(methods=['POST'],
                         manual_parameters=[lang, use_default_model, custom_model_id],
                         request_body=TextSerializer)
    @action(detail=False, methods=['POST'], name='Named Entity Recognition')
    def ner(self, request):
        text = request.data['text']
        lang = request.query_params['lang']
        entities = []
        if lang == 'zh':
            entities += lac_ner(text)
            entities += RegexMatcher.match_chinese_mobile_phone_number(text, offset=False)
            entities += RegexMatcher.match_chinese_id_card_number(text, offset=False)
        return Response({"status_code": 200,
                         "data": {'entities': entities}})

    @swagger_auto_schema(methods=['POST'],
                         manual_parameters=[lang, threshold, mode, return_all],
                         request_body=OneMatchManySerializer)
    @action(detail=False, methods=['POST'],
            url_path='fuzzy-matching/one-match-many',
            name='Fuzzy Match for one to many.')
    def one_match_many(self, request):
        source = request.data['source']
        targets = request.data['targets']
        lang = request.query_params['lang']
        threshold = float(request.query_params['threshold'])
        mode = request.query_params['mode']
        return_all = request.query_params['return_all']
        result = FuzzyMatcher.one_match_many(source=source, targets=targets, lang=lang,
                                             threshold=threshold, mode=mode, return_all=return_all)
        return Response({"status_code": 200,
                         "data": result})

    @swagger_auto_schema(methods=['POST'],
                         manual_parameters=[lang, threshold, mode, return_all],
                         request_body=ManyMatchOneSerializer)
    @action(detail=False, methods=['POST'],
            url_path='fuzzy-matching/many-match-one',
            name='Fuzzy Match for one to many.')
    def many_match_one(self, request):
        sources = request.data['sources']
        target = request.data['target']
        lang = request.query_params['lang']
        threshold = float(request.query_params['threshold'])
        mode = request.query_params['mode']
        return_all = request.query_params['return_all']
        result = FuzzyMatcher.many_match_one(sources=sources, target=target, lang=lang,
                                             threshold=threshold, mode=mode, return_all=return_all)
        return Response({"status_code": 200,
                         "data": result})

    @swagger_auto_schema(methods=['POST'],
                         manual_parameters=[lang, threshold, return_all],
                         request_body=SearchSimilaritySerializer)
    @action(detail=False, methods=['POST'],
            url_path='search-similarity',
            name='Search similarity.')
    def search_similarity(self, request):
        source = request.data['source']
        targets = request.data['targets']
        lang = request.query_params['lang']
        threshold = float(request.query_params['threshold'])
        return_all = request.query_params['return_all']
        result = search_similarity(source=source, targets=targets, lang=lang,
                                   threshold=threshold, return_all=return_all)
        return Response({"status_code": 200,
                         "data": result})